/***********************************************************************/
/*  flash_test.c                                            2000-06-20 */
/*                                                                     */
/*                          Flash test                                 */
/*                                                                     */
/* This software demonstrates the functionality and use of the on-chip */
/* flash self-programming capability of MSP430 devices.                */
/* To demonstrate this feature the following program erases and writes */
/* to the entire information memory starting at address 1000h, which   */
/* is part of every MSP430 flash-based device.                         */
/*                                                                     */
/* There are two different methods to program flash on MSP430:         */
/* A) Running the flash program function out of RAM                    */
/*    necessary functions can be found in file flash_ram.s43           */
/* B) direct Flash programming                                         */
/*    necessary functions can be found in file flash_idle.s43          */
/*                                                                     */ 
/* Note 1: project must include assembly file "flash_var.s43", because */
/* in this file some variables at absolute address are defined, which  */
/* are used in this software.                                          */
/*                                                                     */
/* Note 2: Since all interrupt vectors are unavailable during flash    */
/* programming, all interrupts must be disabled.                       */
/*                                                                     */
/* Anton Muehlhofer                     Texas Instruments Incorporated */
/***********************************************************************/

#include <msp430x14x.h>

#include  "flash_prog.h"    /* flash programming function prototypes */


#define   FSEG_A 0x01080    /* Flash Segment A start address */
#define   FSEG_B 0x01000    /* Flash Segment B start address */


/***********************************************************************/
/* main */
/***********************************************************************/
void main( void )
{
 char *ptr;                 /* pointer to byte (8 bit)*/
 char ch;                   /* byte variable */
 int *ptrw;                 /* pointer to word (16 bit) */
 int  w;                    /* word variable */

// === Initialize system ================================================
 IFG1=0;                    /* clear interrupt flag1 */
 WDTCTL=0x5A80;             /* stop WD */
 DCOCTL=0x60;               /* DCO frequency ~750 kHz (RSEL=4) */
 BCSCTL1=0x04;
 FCTL2=0x0A542;             /* select MCLK/3 for flash timing */
 _DINT();                   /* disable all interrupts to protect CPU */
                            /* during programming from system crash */
 IE1=0x0;                   /* disable all NMI-Interrupt sources */

// === Erase flash info memory ========================================== 
 Flash_clr( (int *) FSEG_A );  /* erase segment A */
 Flash_clr( (int *) FSEG_B );  /* erase segment B */

 _NOP();                       /* set Breakpoint */

// === program dedicated addresses in Flash information memory =========
 ptr = (char *) 0x01000;
 FCTL3 = 0x0A500;           /* Lock = 0 */
 FCTL1 = 0x0A540;           /* WRT = 1 */
 *ptr++ = 0x055;            /* program byte at address 0x01000 */
 FCTL1 = 0x0A500;           /* WRT = 0 */
 FCTL3 = 0x0A510;           /* Lock = 1 */
 
 _NOP();                    /* set Breakpoint */

// === Erase flash info memory ========================================== 
 Flash_clr( (int *) FSEG_A );  /* erase segment A */
 Flash_clr( (int *) FSEG_B );  /* erase segment B */

 _NOP();                    /* set Breakpoint */

// === program segment A bytewise =======================================
 ptr = (char *) FSEG_A;   
 for(ch=0;ch<128;ch++)
   Flash_wb( ptr++, ch );

 _NOP();                      /* set Breakpoint */
 
// === program segment B wordwise =======================================
 ptrw = (int *) FSEG_B;  
 for(w=0;w<64;w++)
   Flash_ww( ptrw++, w );

 _NOP();                      /* set Breakpoint */

// === erase segment B ==================================================
 Flash_clr( (int *) FSEG_B ); 

 _NOP();                      /* set Breakpoint */

// === copy segment A into B with verify ================================
 Flash_copy( (int *) FSEG_A, (int *) FSEG_B);

 _NOP();                      /* set Breakpoint */

 while( 1 );
} 
// === end of main =====================================================


/***********************************************************************/
/*                          Flash_copy                                 */
/***********************************************************************/
/* copies segment A into B with verify and returns 0 if operation was  */
/* successful or address of flash memory, which couldn't be programmed.*/
/* This function tries to program one word up to 5 times. If the       */
/* verify still fails, the next word will be programmed.               */
/***********************************************************************/
int Flash_copy( int *source_ptr, int *dest_ptr )
{
 unsigned int cntr;
 unsigned int w;
 int error=0;
 
 for( w=0; w<64; w++ )
 {
  cntr=0;
  do                             /* tries to program word 5 times  */
  {
    Flash_ww( dest_ptr, *source_ptr );
    cntr++;
  } while( (*dest_ptr != *source_ptr) | (cntr > 4 ) );
  if( cntr > 4) error=(unsigned int) dest_ptr;
  dest_ptr++;
  source_ptr++;
 }
 return( error );
}
// === end of function ==================================================
